package gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EtchedBorder;

import filter.Filter;
import filter.TokenContainer;

public class gui_main {
	private static JPanel statusPanel;
	private static JPanel mainPanel;
	private static JPanel tokenTable;
	private static JPanel resultOverview;
	private static JLabel numOfEntriesSpam = new JLabel("1000");
	private static JLabel numOfEntriesHam = new JLabel("500");

	private static int numberOfDifferentResultForOverview = 5;
	
	private static JFrame frmPleaseWait = new JFrame("Bitte warten...");
	//	Create new Filter and hook it up to the DB
	private static String delimiter = "\\s+[.,;]*\\s*";
	private static Filter testFilter;
	
	private gui_main() {
		initComponents();
	}
	
	
	private void initComponents() {
		JTabbedPane tabbedPane = new JTabbedPane();
		
		JFrame mainFrame = new JFrame();
		mainFrame.setTitle("Bayes Filter GUI v0.3");
		mainFrame.addWindowListener(new MyWindowAdapter());
		mainFrame.getContentPane().setLayout(new BorderLayout());
		
		JMenuBar menuBar = buildMenu.getInstance().getMenuBar();
		mainFrame.setJMenuBar(menuBar);
		
		statusPanel = buildStatusPanel();
		statusPanel.setBorder(new EtchedBorder());
		mainFrame.getContentPane().add(statusPanel,BorderLayout.SOUTH);

		/* first tabbed pane  is the 'main panel'*/
		mainPanel = buildMainPanel.getInstance().getMainPanel();
		tabbedPane.addTab("Analyse",mainPanel);
		
		/* second tabbed pane is the 'tokenTable panel*/
		tokenTable = buildTokenTable.getInstance().getTokenTable();
		tabbedPane.addChangeListener(new tabbedChangeListener());
		tabbedPane.addTab("Datenbank-Inhalt",tokenTable);
		
		/* third tabbed pane is the 'different results overview panel*/
		resultOverview = buildResultOverviewTable.getInstance().getOverviewTable();
		tabbedPane.addChangeListener(new tabbedChangeListener());
		tabbedPane.addTab("Ergebnisse verschiedener Genauigkeit",resultOverview);
		
		//mainFrame.getContentPane().add(mainPanel,BorderLayout.CENTER);
		mainFrame.getContentPane().add(tabbedPane,BorderLayout.CENTER);
		mainFrame.setResizable(false);
		mainFrame.setSize(800,600);
		mainFrame.setVisible(true);

//		construct please-wait frame
		frmPleaseWait.setSize(200,50);
		frmPleaseWait.setLocation((int)mainFrame.getWidth()/2, (int)mainFrame.getHeight()/2);
		/*
		 * After building the gui we start here with the filter's work
		 */
		 try {
			 testFilter = new Filter("jdbc:mysql://localhost/test", "filter", "filter", delimiter);
			 //testFilter = new Filter("jdbc:mysql://localhost/test", "nille", "", delimiter);
		 }catch (SQLException e) {
			 // TODO Auto-generated catch block
			 e.printStackTrace();
		 } catch (ClassNotFoundException e) {
			 // TODO Auto-generated catch block
			 e.printStackTrace();
		 }
	}
	
	private JPanel buildStatusPanel() {
		JPanel statusPanel = new JPanel();
		JLabel EntriesSpam = new JLabel("Anzahl der Spams: ");
		JLabel EntriesHam = new JLabel("Anzahl der Hams: ");
		
		statusPanel.setLayout(new FlowLayout());
		statusPanel.add(EntriesSpam);
		statusPanel.add(numOfEntriesSpam);
		statusPanel.add(EntriesHam);
		statusPanel.add(numOfEntriesHam);
		
		return statusPanel;
	}
	
	/*
	 * Public section begins here ...
	 * */
	public static void sanitizeDatabase(){
		try {
			testFilter.sanitizeDatabase();
			int totals[] = testFilter.queryDBForTotalCounts();
			numOfEntriesSpam.setText(Integer.toString(totals[0]));
			numOfEntriesHam.setText(Integer.toString(totals[1]));
			}catch (SQLException e) {
			// TODO Auto-generated catch block
			System.out.println("Error while sanitizing DB in gui_main: " + e.getMessage());
		}
	}
	
	public static void learnTextAsSpam() {
		if(buildMainPanel.getInstance().getTextAreaContent() != null) {
			testFilter.trainFilter(buildMainPanel.getInstance().getTextAreaContent(), "spam");
		}
		int totals[] = testFilter.queryDBForTotalCounts();
		numOfEntriesSpam.setText(Integer.toString(totals[0]));
				
	}
	
	public static void learnTextAsHam() {
		if(buildMainPanel.getInstance().getTextAreaContent() != null) {
			testFilter.trainFilter(buildMainPanel.getInstance().getTextAreaContent(), "ham");
		}
		int totals[] = testFilter.queryDBForTotalCounts();
		numOfEntriesHam.setText(Integer.toString(totals[1]));
	}
	
	public static void analyzeText() {
		double result;
		// show please-wait frame
		frmPleaseWait.setVisible(true);
		
		if(buildMainPanel.getInstance().getTextAreaContent() != null) {
			result = testFilter.evaluateInput(buildMainPanel.getInstance().getTextAreaContent(),buildMainPanel.getInstance().getNumberOfEvaluatedTokens());
			if(result > 0.95) {
				buildMainPanel.getInstance().txtIsSpam("SPAM zu " + Double.toString(result * 100) + " Prozent");
			}else{
				buildMainPanel.getInstance().txtIsHam("SPAM zu " + Double.toString(result * 100) + " Prozent");
			}
			/* Calculate spammyness for graph and store data in array for Ppoup-Table*/
			Iterator it = testFilter.queryFilter(buildMainPanel.getInstance().getTextAreaContent());
			Vector popupTableVector = new Vector();
			//buildTokenTable.getInstance().setTokenTable(tableData);
			TokenContainer tmpContainer;
			final int[] spammyness = new int[] {0,0,0,0,0,0,0,0,0,0};
			System.out.println(buildMainPanel.getInstance().getNumberOfEvaluatedTokens());
			while(it.hasNext()) {
				tmpContainer = (TokenContainer)it.next();
				spammyness[ (int)(tmpContainer.getSpammyness() * 10 )]++;
				popupTableVector.add(tmpContainer);
			}
			// show please-wait frame
			frmPleaseWait.setVisible(false);
			
			/*create new thread for drawing graph*/
			new Thread() {
	            public void run() {
	            	buildMainPanel.getInstance().drawGraph(spammyness);
	            }
	          }.start();
	        if(buildMainPanel.getInstance().getAllowPopup()) {
	        		PopupTokenTable popup = new PopupTokenTable(popupTableVector);
	        		popup.showTable();
	        }
		}
		
	}
	
	public static void buildNewTokenTable() {
		Iterator test = testFilter.queryDBForInterestingTokens(-1);
		Vector tokenTableVector = new Vector();
		int i = 0;
		
		while(test.hasNext()){
			TokenContainer temp = (TokenContainer)test.next();
			tokenTableVector.add(temp);
		}
		buildTokenTable.getInstance().setTokenTable(tokenTableVector);
	}
	
		
	public static void buildResultOverviewTable() {
		String[][] tableData = new String[numberOfDifferentResultForOverview][2];
		double result;
		
		for(int i = 1; i<= numberOfDifferentResultForOverview; i++) {
			result = testFilter.evaluateInput(buildMainPanel.getInstance().getTextAreaContent(),(i*5));
			tableData[(i-1)][0] = (i*5) + " Token bewertet "; 
			tableData[(i-1)][1] = Double.toString(result);
		}
		buildResultOverviewTable.getInstance().setTokenTable(tableData);
	}
	
/*	public static String[][] getTokenDataFromActualFilter(int i) {
		int numberOfTokensToDisplay = i;
		String[][] tableData = new String[0][0];;
		if(numberOfTokensToDisplay > 0) {
			Iterator test = testFilter.queryDBForInterestingTokens(buildMainPanel.getInstance().getNumberOfEvaluatedTokens());
			tableData = new String[buildMainPanel.getInstance().getNumberOfEvaluatedTokens()][5];
			i = 0;
			
			while(test.hasNext()){
				TokenContainer temp = (TokenContainer)test.next();
				tableData[i][0] = temp.toString();
				tableData[i][1] = Integer.toString(temp.getAppearancesInHam());
				tableData[i][2] = Integer.toString(temp.getAppearancesInSpam());
				tableData[i][3] = Double.toString(temp.getSpammyness());
				tableData[i][4] = Double.toString(temp.getInterestingness());
				i++;
			}
		}
		return tableData;
	}
	*/
	public static void main(String[] args) {
		gui_main gm = new gui_main();
		int totals[] = testFilter.queryDBForTotalCounts();
		numOfEntriesSpam.setText(Integer.toString(totals[0]));
		numOfEntriesHam.setText(Integer.toString(totals[1]));
	}
}
